﻿using PTMIBase.Extensions.Attributes;
using PTMIBase.Globals;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PTMIBase.MIForm
{
    public class DebugBaseFormHelper<TEnum> where TEnum : struct, Enum
    {
        // 动态子类型映射表
        private static readonly Dictionary<TEnum, Type> SubTypeMap = new Dictionary<TEnum, Type>();

        /// <summary>
        /// 初始化子类型映射表（支持动态注册）
        /// </summary>
        public static void InitializeSubTypeMap(Dictionary<TEnum, Type> subTypeMapping)
        {
            SubTypeMap.Clear();
            foreach (var mapping in subTypeMapping)
            {
                SubTypeMap[mapping.Key] = mapping.Value;
            }
        }

        /// <summary>
        /// 获取指定枚举值对应的子类型列表
        /// </summary>
        public static List<dynamic> GetSubTypesForEnum(TEnum enumValue)
        {
            var subTypes = new List<dynamic>();
            if (SubTypeMap.TryGetValue(enumValue, out Type enumType))
            {
                foreach (var field in enumType.GetFields(BindingFlags.Public | BindingFlags.Static))
                {
                    var tradeAttr = field.GetCustomAttribute<TradeAttribute>();
                    if (tradeAttr != null)
                    {
                        subTypes.Add(new
                        {
                            Code = tradeAttr.Code,
                            Description = tradeAttr.Name
                        });
                    }
                }
            }
            return subTypes;
        }

        /// <summary>
        /// 获取主类型列表（基于当前枚举类型）
        /// </summary>
        public static List<dynamic> GetMainTypes()
        {
            var mainTypes = new List<dynamic>();
            foreach (var field in typeof(TEnum).GetFields(BindingFlags.Public | BindingFlags.Static))
            {
                var tradeAttr = field.GetCustomAttribute<TradeAttribute>();
                if (tradeAttr != null)
                {
                    mainTypes.Add(new
                    {
                        Code = tradeAttr.Code,
                        Description = tradeAttr.Name
                    });
                }
            }
            return mainTypes;
        }

        /// <summary>
        /// 将字符串解析为指定的枚举值
        /// </summary>
        public static TEnum GetEnumValue(string value)
        {
            if (Enum.TryParse(value, true, out TEnum result))
            {
                return result;
            }

            throw new ArgumentException($"无法将 '{value}' 转换为枚举类型 {typeof(TEnum).Name}。");
        }

        /// <summary>
        /// 更新主类型
        /// </summary>
        /// <param name="cbx"></param>
        public static void UpdateComboBoxMainType(ComboBox cbx)
        {
            var items = GetMainTypes();
            cbx.DataSource = items;
            cbx.DisplayMember = "Description";
            cbx.ValueMember = "Code";
        }

        /// <summary>
        /// 更新子类型
        /// </summary>
        /// <param name="selectedType"></param>
        /// <param name="cbxSubType"></param>
        public static void UpdateComboBoxSubType(TEnum selectedType, ComboBox cbxSubType)
        {
            var subTypes = GetSubTypesForEnum(selectedType);
            cbxSubType.DataSource = subTypes;
            cbxSubType.DisplayMember = "Description";
            cbxSubType.ValueMember = "Code";
        }

        /// <summary>
        /// 下拉框值变化处理
        /// </summary>
        /// <param name="cbxMainType"></param>
        /// <param name="cbxSubType"></param>
        public static void UpdateSubTypeAfterCbxMainTypeChanged(ComboBox cbxMainType, ComboBox cbxSubType)
        {
            if (cbxMainType.SelectedValue is string selectedValue)
            {
                // 根据 ValueMember 的值找到对应的 ShellBusTypeEnum
                var selectedType = GetEnumValue(selectedValue);
                UpdateComboBoxSubType(selectedType, cbxSubType);
            }
        }

        /// <summary>
        /// 初始化ComBox
        /// </summary>
        /// <param name="subTypeMapping"></param>
        /// <param name="cbxMainType"></param>
        /// <param name="cbxSubType"></param>
        public static void InitCombox(Dictionary<TEnum, Type> subTypeMapping, ComboBox cbxMainType, ComboBox cbxSubType)
        {
            // 初始化子类型映射表
            InitializeSubTypeMap(subTypeMapping);
            //更新主类型
            UpdateComboBoxMainType(cbxMainType);
            //根据选择的主类型更新子类型
            UpdateSubTypeAfterCbxMainTypeChanged(cbxMainType, cbxSubType);
        }

        
    }
}
